package org.apidesign.apifest08.currency;



/**
 * Creates {@link Convertor} implementations.
 * @author lukas
 *
 */
public class ConvertorFactory {
	private ConvertorFactory()
	{
		//nothing
	}
		
	/**
	 * Creates {@link Convertor} that converts from sourceEquivalent.currency to destinationEquivalent.currency. 
	 * Exchange rate is set as equivalents. It means if you want to create USD to CZK convertor where USD1 = CZK17 
     * call createConvertor(new MoneyImpl(1, USD), new MoneyImpl(17, CZK)). Convertor created by this method 
     * rounds the result to two decimal places.
	 * @param sourceEquivalent
	 * @param destinationEquivalent
	 * @return
	 */
	public static final ExtendedConvertor createConvertor(Money sourceEquivalent, Money destinationEquivalent)
	{
		return mergeConvertors(
				new DefaultConvertor(sourceEquivalent, destinationEquivalent),
				new DefaultConvertor(destinationEquivalent ,sourceEquivalent)
		);
	}
	
	/**
	 * Merges convertors. The resulting convertor has ability to do all conversions that its underlying 
	 * convertors could do. No consistency validation is done, inconsistent input will result in a convertor with
	 * inconsistent behavior.
	 * @param convertors
	 * @return
	 */
	public static final ExtendedConvertor mergeConvertors(ExtendedConvertor... convertors)
	{
		return new CompositeConvertor(convertors);
	}
	

}
